package client;

import interfaces.*;
import java.rmi.*;
import java.util.*;

/**********************************************************************/
/* This is the client that will send tasks to the servers.            */
/* It will                                                            */
/*                                                                    */
/*    1. Get a list of available servers (via LBLookup service)       */
/*    2. Determine the less loaded server (via Executor service)      */
/*    3. Send a task to the less loaded server (via Executor service) */
/*                                                                    */
/**********************************************************************/

// This is an exemple of a task : it prints "hello world!" on the screen 
class PrintHello implements Task {
    public PrintHello() {
    }
    
    public Object run() {
	System.out.println("Hello world !");
	for (long l = 0; l < 100000000; l++);
	return null;
    }
}


// This is the client that searches the best server and send it a task
public class Client {

    // Method called by the VM : instanciates a Client and sends the
    // PrintHello to a server
    public static void main(String[] s) {
	Client cl = new Client();
	cl.executeTask(new PrintHello());
    }

   
    public Client() {
    }

    
    // Searches the best server and sends it a task
    // Returns : null in case of problem (no servers or server error)
    //           Object that is the result of the task's calculation
    public Object executeTask (Task task) {
	try {
	    Distributor d = (Distributor)  
                Naming.lookup(NamingCst.Server + NamingCst.DistributorName);
            return d.forwardTask(task);
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return null;
    }
}
