/*******************************************************************/
/* This is the server that will forward tasks that are sent by     */
/* the clients.                                                    */
/*                                                                 */
/*                                                                 */
/* IMPLEMENTATION OF THE DISTRIBUTOR INTERFACE                     */
/*******************************************************************/


package server;
import interfaces.*;

import java.rmi.*;
import java.rmi.server.*;
import java.util.*;
 

public class DServer extends UnicastRemoteObject
                    implements Distributor
{
    private Vector mExecutors;


    DServer() throws RemoteException {
        super();
        mExecutors = new Vector();

        try {
            Naming.rebind(NamingCst.Server + 
                          NamingCst.DistributorName, this);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] s) {
        try {
            new DServer();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    //-------------------------------------------------------------------
    // Implementation of the Distributor interface

    public Object forwardTask(Task t) throws RemoteException {
        Executor bestServer = null;

        if (mExecutors.size()>0) {
            int bestLoad = 0;
            for (int i = 0; i < mExecutors.size(); i++) {                
                Executor e = (Executor) mExecutors.get(i);
                int  load  = e .getLoad();
                if (i == 0 || load < bestLoad) {
                    bestLoad   = load;
                    bestServer = e;
                }
            }
        } else {
            throw new RemoteException("No servers available.");
        }

        return bestServer.execute(t);
    }


    public synchronized void register(Executor e) throws RemoteException {
        mExecutors.add(e);
        System.out.println("DServer: Number of Servers is " + mExecutors.size());
    }
    public synchronized void unregister(Executor e) throws RemoteException {
        mExecutors.remove(e);
        System.out.println("DServer: Number of Servers is " + mExecutors.size());
    }
 

}
