/*******************************************************************/
/* This is the server that will execute tasks that are sent by     */
/* the clients.                                                    */
/*                                                                 */
/* When it is instanciated, it will register itself with the       */
/* distributor service which maintains a list of                   */
/* Executors.                                                      */
/*                                                                 */
/* IMPLEMENTATION OF THE EXECUTOR INTERFACE                        */
/*******************************************************************/


package server;
import interfaces.*;

import java.rmi.*;
import java.rmi.server.*;
 

public class Server extends UnicastRemoteObject
                    implements Executor
{

    // Keep track of the number of tasks that are being executed
    private int mLoad;
    private int   mId; // Only for debugging

    // Instanciate -> register with lblookup service
    public Server(int id) throws RemoteException {
	super();
	mLoad = 0;
        mId   = id;
	
	try {
	    Distributor d = (Distributor)  
                Naming.lookup(NamingCst.Server + NamingCst.DistributorName);
	    d.register(this);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    // Destruction --> deregister with lblookup service
    protected void finalize () {
	System.out.println("Cleaning up...");
	try {
	    Distributor d = (Distributor)  
                Naming.lookup(NamingCst.Server + NamingCst.DistributorName);
	    d.unregister(this);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }


    // Method called by the VM : instanciate a server
    public static void main(String[] s) {
        System.runFinalizersOnExit(true);

	try {
	    Executor server = new Server(0);
	    System.out.println("Server running. Press ENTER to stop.");

	    // Wait for keystroke
	    int a = System.in.read();

	    // Makes finalize() get called 
	    System.exit(0);
	    
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    // Changes the load of the server
    private synchronized void addLoad(int i) {
	mLoad += i;
	System.out.println("Server load is now " + mLoad);
    }


    //-------------------------------------------------------------------
    // Implementation of the Executor interface

    public Object execute(Task task) throws RemoteException {
	addLoad(1);
	Object ret = task.run();
	addLoad(-1);

	return ret;
    }

    public synchronized int getLoad() {
	return mLoad;
    }
}
